\name{lvsplot}
\alias{lvsplot}
\title{Plot the latent variables from an boral model} 
\description{Construct a 1-D index plot or 2-D scatterplot of the latent variables from a fitted boral model.}

\usage{lvsplot(x, jitter = FALSE, a = 1, newplot = TRUE, ...) }

\arguments{
  \item{x}{An object for class "boral".}

  \item{jitter}{If \code{jitter = TRUE}, then some jittering is applied so that points on the plots do not overlap exactly (which can often occur with discrete data and small sample sizes). Please see \code{\link{jitter}} for its implementation.}
     
  \item{a}{Default parameter used in cex. Graphical options are adjusted as \cr \code{par(cex = a, cex.axis = a, cex.lab = a+0.5, cex.main = a+0.5, ...)}. Defaults to 1.}
      
  \item{newplot}{Construct plots on a new graphics device? Defaults to TRUE.}
   
   \item{...}{Additional graphical options to be included in \code{par}.}
}

\details{
Two plots are produced, based on the posterior medians and posterior means of the latent variables respectively. The points in each plot are labeled using the row index of the response matrix \eqn{y}. 

If the fitted model does not include covariates (\code{X = NULL}), the plots can be interpreted in the exactly same manner as unconstrained ordination plots constructed from methods such as Nonmetric Multi-dimensional Scaling (NMDS, Kruskal, 1964) and Correspondence Analysis (CA, Hill, 1974). with multivariate abundance data for instance, where the response matrix \eqn{y} consists of \eqn{n} sites and \eqn{p} species, the ordination plots can be studied to look for possible clustering of sites, location and/or dispersion effects, an arch pattern indicative of some sort species succession over an environmental gradient, and so on.
}

\author{
Francis K.C. Hui \email{fhui28@gmail.com}
}

\references{
\itemize{
\item Hill, M. O. (1974). Correspondence analysis: a neglected multivariate method. Applied statistics, 23, 340-354.
\item Kruskal, J. B. (1964). Nonmetric multidimensional scaling: a numerical method. Psychometrika, 29, 115-129.
}
}

\examples{
\dontrun{
library(mvabund) ## Load a dataset from the mvabund package
data(spider)
y <- spider$abun
n <- nrow(y); p <- ncol(y); 

spider.fit.nb <- boral(y, family = "negative.binomial", num.lv = 2, 
     row.eff = TRUE, save.model = FALSE, calc.ics = FALSE)

## Three clusters are observable with site 25 roughly in the middle.
lvsplot(spider.fit.nb) 
}
}